@@ -18,13 +18,13 @@ graphics package, and as LaTeX/TikZ source for direct embedding into documents o
 user-tweaking of the generated output.
 
 TODOs:
- * Identify HERWIG decay matrix element to use in ISAWIG
  * Split writeSLHA into writeSLHA{Blocks,Decays}
+ * Identify HERWIG decay matrix element to use in ISAWIG
  * Handle RPV SUSY in ISAWIG
 """
 
 __author__ = "Andy Buckley <andy.buckley@cern.ch"
-__version__ = "1.2.4"
+__version__ = "1.2.5"
 
 
 def _autotype(var):
@@ -39,10 +39,10 @@ def _autotype(var):
     except ValueError:
         return var
 
-def _autostr(var):
+def _autostr(var, precision=8):
     """Automatically numerical types to the right sort of string."""
     if type(var) is float:
-        return "%e" % var
+        return ("%." + str(precision) + "e") % var
     return str(var)
 
 
@@ -112,7 +112,7 @@ class Decay(object):
         return cmp(other.br, self.br)
 
     def __str__(self):
-        return "%e %s" % (self.br, self.ids)
+        return "%.8e %s" % (self.br, self.ids)
 
     def __repr__(self):
         return self.__str__()
@@ -148,9 +148,9 @@ class Particle(object):
     def __str__(self):
         s = str(self.pid)
         if self.mass is not None:
-            s += " : mass = %e GeV" % self.mass
+            s += " : mass = %.8e GeV" % self.mass
         if self.totalwidth is not None:
-            s += " : total width = %e GeV" % self.totalwidth
+            s += " : total width = %.8e GeV" % self.totalwidth
         for d in self.decays:
             if d.br > 0.0:
                 s += "\n  %s" % d
@@ -277,10 +277,12 @@ def writeSLHAFile(spcfilename, blocks, decays, **kwargs):
 # TODO: Split writeSLHA into writeSLHA{Blocks,Decays}
 
 
-def writeSLHA(blocks, decays, ignorenobr=False):
+def writeSLHA(blocks, decays, ignorenobr=False, precision=8):
     """
     Return an SLHA definition as a string, from the supplied blocks and decays dicts.
     """
+    fmte = "%." + str(precision) + "e"
+
     sep = "   "
     out = ""
     def dict_hier_strs(d, s=""):
@@ -294,18 +296,18 @@ def writeSLHA(blocks, decays, ignorenobr=False):
     for bname, b in sorted(blocks.iteritems()):
         namestr = b.name
         if b.q is not None:
-            namestr += " Q= %e" % float(b.q)
+            namestr += (" Q= " + fmte) % float(b.q)
         out += "BLOCK %s\n" % namestr
         for s in dict_hier_strs(b.entries):
             out += sep + s + "\n"
         out += "\n"
     ## Decays
     for pid, particle in sorted(decays.iteritems()):
-        out += "DECAY %d %e\n" % (particle.pid, particle.totalwidth or -1)
+        out += ("DECAY %d " + fmte + "\n") % (particle.pid, particle.totalwidth or -1)
         for d in sorted(particle.decays):
             if d.br > 0.0 or not ignorenobr:
                 products_str = "   ".join(map(str, d.ids))
-                out += sep + "%e" % d.br + sep + "%d" % len(d.ids) + sep + products_str + "\n"
+                out += sep + fmte % d.br + sep + "%d" % len(d.ids) + sep + products_str + "\n"
         out += "\n"
     return out
 
@@ -657,7 +659,7 @@ def writeISAWIGFile(isafilename, blocks, decays, **kwargs):
     f.close()
 
 
-def writeISAWIG(blocks, decays, ignorenobr=False):
+def writeISAWIG(blocks, decays, ignorenobr=False, precision=8):
     """
     Return an ISAWIG definition as a string, from the supplied blocks and decays dicts.
 
@@ -667,6 +669,7 @@ def writeISAWIG(blocks, decays, ignorenobr=False):
     If the ignorenobr parameter is True, do not write decay entries with a
     branching ratio of zero.
     """
+    fmte = "%." + str(precision) + "e"
 
     masses = blocks["MASS"].entries
 
@@ -688,7 +691,7 @@ def writeISAWIG(blocks, decays, ignorenobr=False):
                 lifetime = 1.0/(width * 1.51926778e24) ## lifetime in seconds == hbar/width in GeV
         except:
             pass
-        massout += "%d %e %e\n" % (pdgid2herwigid(pid), masses[pid], lifetime)
+        massout += ("%d " + fmte + " " + fmte + "\n") % (pdgid2herwigid(pid), masses[pid], lifetime)
     out += "%d\n" % massout.count("\n")
     out += massout
 
@@ -726,7 +729,7 @@ def writeISAWIG(blocks, decays, ignorenobr=False):
             if len(d.ids) == 3:
                 # TODO: How to determine the conditions for using 200 and 300 MEs? Enumeration of affected decays?
                 pass
-            decayout += "%d %e %d " % (hwid, d.br, nme)
+            decayout += "%d " + fmte + " %d " % (hwid, d.br, nme)
 
             def is_quark(pid):
                 return (abs(pid) in range(1, 7))
@@ -872,25 +875,25 @@ def writeISAWIG(blocks, decays, ignorenobr=False):
 
     ## Now the SUSY parameters
     ## TANB, ALPHAH:
-    out += "%e %e\n" % (blocks["MINPAR"].entries[3], blocks["ALPHA"].entries)
+    out += (fmte + " " + fmte + "\n") % (blocks["MINPAR"].entries[3], blocks["ALPHA"].entries)
     ## Neutralino mixing matrix
     nmix = blocks["NMIX"].entries
     for i in xrange(1, 5):
-        out += "%e %e %e %e\n" % (nmix[i][1], nmix[i][2], nmix[i][3], nmix[i][4])
+        out += (fmte + " " + fmte + " " + fmte + " " + fmte + "\n") % (nmix[i][1], nmix[i][2], nmix[i][3], nmix[i][4])
     ## Chargino mixing matrices V and U
     vmix = blocks["VMIX"].entries
-    out += "%e %e %e %e\n" % (vmix[1][1], vmix[1][2], vmix[2][1], vmix[2][2])
+    out += (fmte + " " + fmte + " " + fmte + " " + fmte + "\n") % (vmix[1][1], vmix[1][2], vmix[2][1], vmix[2][2])
     umix = blocks["UMIX"].entries
-    out += "%e %e %e %e\n" % (umix[1][1], umix[1][2], umix[2][1], umix[2][2])
+    out += (fmte + " " + fmte + " " + fmte + " " + fmte + "\n") % (umix[1][1], umix[1][2], umix[2][1], umix[2][2])
     # THETAT,THETAB,THETAL
     import math
-    out += "%e %e %e\n" % (math.acos(blocks["STOPMIX"].entries[1][1]),
-                           math.acos(blocks["SBOTMIX"].entries[1][1]),
-                           math.acos(blocks["STAUMIX"].entries[1][1]))
+    out += (fmte + " " + fmte + " " + fmte + " " + "\n") % (math.acos(blocks["STOPMIX"].entries[1][1]),
+                                                            math.acos(blocks["SBOTMIX"].entries[1][1]),
+                                                            math.acos(blocks["STAUMIX"].entries[1][1]))
     # ATSS,ABSS,ALSS
-    out += "%e %e %e\n" % (blocks["AU"].entries[3][3],
-                           blocks["AD"].entries[3][3],
-                           blocks["AE"].entries[3][3])
+    out += (fmte + " " + fmte + " " + fmte + " " + "\n") % (blocks["AU"].entries[3][3],
+                                                            blocks["AD"].entries[3][3],
+                                                            blocks["AE"].entries[3][3])
     # MUSS == sign(mu)
     out += "%f\n" % blocks["MINPAR"].entries[4]
 
